## $Id: catalog.py 9767 2012-12-05 06:03:12Z uli $
##
## Copyright (C) 2011 Uli Fouquet & Henrik Bettermann
## This program is free software; you can redistribute it and/or modify
## it under the terms of the GNU General Public License as published by
## the Free Software Foundation; either version 2 of the License, or
## (at your option) any later version.
##
## This program is distributed in the hope that it will be useful,
## but WITHOUT ANY WARRANTY; without even the implied warranty of
## MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
## GNU General Public License for more details.
##
## You should have received a copy of the GNU General Public License
## along with this program; if not, write to the Free Software
## Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307 USA
##
"""Components to help cataloging and searching objects.
"""
import grok
from hurry.query.interfaces import IQuery
from hurry.query.query import Query
from zope.catalog.catalog import ResultSet
from zope.catalog.interfaces import ICatalog
from zope.component import getUtility, queryUtility
from zope.interface import implementer
from zope.intid.interfaces import IIntIds
from waeup.kofa.interfaces import (
    IQueryResultItem, IFilteredQuery, IFilteredCatalogQuery)

# not yet used
@implementer(IQuery)
class KofaQuery(Query):
    """A hurry.query-like query that supports also ``apply``.
    """
    def apply(self, query):
        """Get a catalog's BTree set of intids conforming to a query.
        """
        return query.apply()

    def searchResults(self, query):
        """Get a set of ZODB objects conforming to a query.
        """
        results = self.apply(query)
        if results is not None:
            uidutil = getUtility(IIntIds)
            results = ResultSet(results, uidutil)
        return results

grok.global_utility(KofaQuery)

# not yet used
@implementer(IQueryResultItem)
class QueryResultItem(object):
    url = None
    title = None
    description = None

    def __init__(self, context, view):
        self.context = context
        self.url = view.url(context)
        self.title = context.title
        self.description = ''

@implementer(IFilteredQuery)
class FilteredQueryBase(object):
    """A filter to find objects that match certain parameters.
    """
    defaults = dict()

    def __init__(self, **kw):
        self._kw = dict(self.defaults)
        self._kw.update(kw)
        return

    def query(self, context=None):
        err_msg = 'class %s does not implement the query() method.' % (
            self.__class__.__name__, )
        raise NotImplementedError(err_msg)

@implementer(IFilteredCatalogQuery)
class FilteredCatalogQueryBase(FilteredQueryBase):
    cat_name = None

    def query_catalog(self, catalog):
        query = dict()
        for idx_name, value in self._kw.items():
            if not isinstance(value, tuple):
                value = (value, value)
            query[idx_name] = value
        result = catalog.searchResults(**query)
        return result

    def query(self):
        catalog = queryUtility(
            ICatalog, name=self.cat_name, default=None)
        if catalog is None:
            return []
        return self.query_catalog(catalog)
